# coding=UTF-8
from __future__ import print_function, absolute_import, division
import six
from collections import defaultdict
import logging
from sai.basic import Cassandra
import time
from .utils import CounterHelper
import datetime

"""
    Dimensions are     
        - rollup
            - msg_class
"""

ROLLUPS = { # name => seconds
    '5m': 5*60,
    '15m': 15*60,
    '60m': 60*60,
    '3h': 60*60*3,
    '24h': 60*60*24
}

TOTALS = defaultdict(CounterHelper)

def oob_sent():
    p = datetime.datetime.utcfromtimestamp(time.time())
    now_day = datetime.date(year=p.year, month=p.month, day=p.day)

    # We will execute 2 queries and check that

    # you need to be bigger than this to qualify (milliseconds)
    cutoff_point_for = dict(
        (rollname, (time.time() - ROLLUPS[rollname])*1000) for rollname in ROLLUPS.keys()
    )
    metrics = dict((rollname, defaultdict(lambda: 0)) for rollname in ROLLUPS.keys())


    supl_sms_by_owner_24 = defaultdict(lambda: 0)   # supplemental metric - rollup 24h, by owner.

    with Cassandra.cursor() as cur:
        p = cur.execute('SELECT sent_on, msg_class, owner FROM s4obj.oob_messages_sent WHERE day IN (%s, %s)',
                        (now_day, now_day-datetime.timedelta(1)))

    msg_classes_found = set()

    for sent_on, msg_class, owner in p:
        TOTALS[msg_class].feed(sent_on)
        msg_classes_found.add(msg_class)

        for rollname, cutoff in cutoff_point_for.iteritems():
            if sent_on < cutoff:
                continue

            metrics[rollname][msg_class] += 1

            if rollname == '24h' and msg_class == 'sms':
                supl_sms_by_owner_24[owner] += 1

    for rollup, stat in metrics.iteritems():

        for msg_class_name in msg_classes_found:
            yield u'smok4_oob_messages{rollup="%s", msg_class="%s"} %s' % (rollup, msg_class_name,
                                                                           metrics[rollup][msg_class_name])

        for owner, val in supl_sms_by_owner_24.iteritems():
            yield u'smok4_sms_by_owner_24h{owner="%s"} %s' % (owner, val)

    for msg_class in msg_classes_found:
        yield u'smok4_total{msg_class="%s"} %s' % (msg_class, TOTALS[msg_class].value())
